package drew;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.io.Serializable;
import javax.swing.*;
import drew.util.*;

public class Graph extends JPanel implements Serializable
{
  static long serialVersionUID = -4717349660070458116L;

  public static int LINE = 0, BAR = 1, SQUARE = 2, DIAMOND = 3, TRIANGLE = 4;

  private Rectangle graphRect = new Rectangle();

  private Vector plots = new Vector();
  private Vector texts = new Vector();
 

  private Color graphBackgroundColor = Color.white;

  public Color getGraphBackground()
  {
    return graphBackgroundColor;
  }

  public void setGraphBackground(Color c)
  {
    graphBackgroundColor = c;
  }


  private double minX = 0, minY = 0, maxX = 1, maxY = 1;
  public double getMinX()
  {
    return minX;
  }
  public void setMinX(double x)
  {
    minX = x;
  }
  public double getMaxX()
  {
    return maxX;
  }
  public void setMaxX(double x)
  {
    maxX = x;
  }
  public double getMinY()
  {
    return minY;
  }
  public void setMinY(double y)
  {
    minY = y;
  }
  public double getMaxY()
  {
    return maxY;
  }
  public void setMaxY(double y)
  {
    maxY = y;
  }



  public Graph()
  {
    setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
    setPreferredSize(new Dimension(210,200));
  }

  private class Plot implements Serializable
  {
    Column x, y;
    Color color;
    int style;
  }

  private class Text implements Serializable
  {
    String text;
    double x, y;
    Color color;
  }


  public void plot(Column xdata, Column ydata, Color c, int s)
  {
    Plot p = new Plot();
    p.x = xdata;
    p.y = ydata;
    p.color = c;
    p.style = s;
    plots.addElement(p);
    repaint();
  }

  public void plot(Column ydata)
  {
    plot(ydata, Color.black, LINE);
  }

  public void plot(Column ydata, Color c, int s)
  {
    double[] xdata = new double[ydata.size()];
    double space = (double)(maxX-minX)/(double)(xdata.length-1);
    for (int i = 0; i < xdata.length; i++)
      xdata[i] = minX + i*space;
    plot(new ArrayColumn(xdata),ydata,c,s);
  }

  public void plot(String s, double x, double y, Color c)
  {
    Text t = new Text();
    t.x = x;
    t.y = y;
    t.color = c;
    t.text = s;
    texts.addElement(t);
    repaint();
  }
    
  public void clear()
  {
    texts.removeAllElements();
    plots.removeAllElements();
    repaint();
  }
  
  private void transform(double x, double y, Point p)
  {
    p.x = (int) ((x - minX)/(maxX-minX)*graphRect.width);
    p.y = (int) (graphRect.height - (y - minY)/(maxY-minY)*graphRect.height);
  }

  private void drawLineGraph(Graphics g, Column x, Column y)
  {
    Point leftp = new Point(), rightp = new Point(), swap;
    transform(x.ref(0),y.ref(0),leftp);
    for (int i = 0; i < x.size() - 1; i++)
      {
	transform(x.ref(i+1),y.ref(i+1),rightp);
	g.drawLine(leftp.x,leftp.y,rightp.x,rightp.y);
	swap = leftp;
	leftp = rightp;
	rightp = swap;
      }
  }

  private void drawBarGraph(Graphics g, Column x, Column y)
  {
    Point p = new Point();
    int halfw = (int)(0.9*(double)graphRect.width/(double)(x.size()-1)/2.0);
    for (int i = 0; i < x.size(); i++)
      {
	transform(x.ref(i),y.ref(i),p);
	g.fillRect(p.x - halfw,p.y,2*halfw,graphRect.height - p.y);
	//System.out.println(p.y + " " + (graphRect.height-p.y));
      }
  }

  private void drawPointGraph(Graphics g, Column x, Column y, Polygon poly)
  {
    Point p = new Point();
    g = g.create();
    for (int i = 0; i < x.size(); i++)
      {	
        transform(x.ref(i),y.ref(i),p);
	g.translate(p.x,p.y);
	g.drawPolygon(poly);
	g.translate(-p.x,-p.y);
      }
  }

  private void drawOneGraph(Graphics g, Plot p)
  {
    g.setColor(p.color);
    g = g.create(1,0,graphRect.width-1,graphRect.height);

    if (p.x == null || p.y == null || p.x.size() == 0 || p.y.size() == 0)
    {
      System.err.println("Can't draw graph... data is empty");
      return;
    }
    
    if (p.style == LINE)
      drawLineGraph(g,p.x,p.y);
    else if (p.style == BAR)
      drawBarGraph(g,p.x,p.y);
    else if (p.style == SQUARE)
      drawPointGraph(g,p.x,p.y,square);
    else if (p.style == DIAMOND)
      drawPointGraph(g,p.x,p.y,diamond);
    else if (p.style == TRIANGLE)
      drawPointGraph(g,p.x,p.y,triangle);
  }
      
  
  private void drawOneText(Graphics g, Text t)
  {
    g.setColor(t.color);
    g = g.create(1,0,graphRect.width-1,graphRect.height);
    Point p = new Point();
    transform(t.x, t.y, p);
    g.drawString(t.text, p.x, p.y);
  }

  int axesNumberWidth = 30;

  public void setAxesNumberWidth(int w)
  {
    axesNumberWidth = w;
  }

  int axesNumberHeight = 15;

  public void setAxesNumberHeight(int h)
  {
    axesNumberHeight = h;
  }


  private void drawAxes(Graphics g)
  {
    // along the horizontal (x) axis
    g.drawLine(graphRect.x, graphRect.height, graphRect.x + graphRect.width, graphRect.height);
    g.drawString(Double.toString(minX), 
		 graphRect.x, 
		 graphRect.y + graphRect.height + axesNumberHeight);
    g.drawString(Double.toString(maxX), 
		 graphRect.x + graphRect.width - axesNumberWidth,
		 graphRect.y + graphRect.height + axesNumberHeight);

    // along the vertical (y) axis
    g.drawLine(graphRect.x, 0, graphRect.x, graphRect.height);
    g.drawString(Double.toString(maxY),
		 0,
		 axesNumberHeight);
    g.drawString(Double.toString(minY),
		 0,
		 graphRect.height);

  }
  

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g); //paint background    

    graphRect.x = axesNumberWidth;
    graphRect.y = 0;
    graphRect.width = getSize().width - graphRect.x;
    graphRect.height = getSize().height - axesNumberHeight;

    //g.clearRect(0,0,getSize().width,getSize().height);
    
    drawAxes(g);

    g.translate(graphRect.x,graphRect.y);
    g.setColor(graphBackgroundColor);
    g.fillRect(1, 0, graphRect.width, graphRect.height);

    for (int i = 0; i < plots.size(); i++)
      drawOneGraph(g, (Plot) plots.elementAt(i));
    
    for (int i = 0; i < texts.size(); i++)
      drawOneText(g, (Text) texts.elementAt(i));

  }

  /*
  public String getToolTipText(MouseEvent e)
  {
    if ((e.getX() > graphRect.x) && (e.getY() < graphRect.height))
      {
	float coordx = (float)(e.getX() - 25) / graphRect.width * (float)axeswidth
                       + (float)axesx;
	float coordy = (float)(graphRect.height - e.getY()) / graphRect.height
                       *(float)axesheight + (float)axesy;
	return(Float.toString(coordx) + ", " + Float.toString(coordy));
      }
    else return(null);
  }
  */

  static Polygon square = makeSquare();
  static Polygon makeSquare()
  {
    int[] x = new int[4];
    int[] y = new int[4];
    x[0] = -2; y[0] = -2;
    x[1] = 2; y[1] = -2;
    x[2] = 2; y[2] = 2;
    x[3] = -2; y[3] = 2;
    return(new Polygon(x,y,4));
  }

  static Polygon diamond = makeDiamond();
  static Polygon makeDiamond()
  {
    int[] x = new int[4];
    int[] y = new int[4];
    x[0] = 0; y[0] = 2;
    x[1] = 2; y[1] = 0;
    x[2] = 0; y[2] = -2;
    x[3] = -2; y[3] = 0;
    return(new Polygon(x,y,4));
  }
    
  static Polygon triangle = makeTriangle();
  static Polygon makeTriangle()
  {
    int[] x = new int[3];
    int[] y = new int[3];
    x[0] = -2; y[0] = 2;
    x[1] = 2; y[1] = 2;
    x[2] = 0; y[2] = -2;
    return(new Polygon(x,y,3));
  }

  /*  

  public static void main(String args[])
  {
    Graph graphPanel = new Graph();
    graphPanel.setPreferredSize(new Dimension(200,100));
    //graphPanel.setBackgroundColor(Color.black);

    JFrame window = new JFrame();
    window.setContentPane(graphPanel);
    window.pack();
    window.setVisible(true);

    double[] x = new double[3];
    double[] y = new double[3];
    x[0] = 0.0; y[0] = 0.0;
    x[1] = 0.5; y[1] = 0.25;
    x[2] = 1.0; y[2] = 1.0 ;

    double[] x1 = new double[3];
    double[] y1 = new double[3];
    y1[0] = 1.0;
    y1[1] = 0.8;
    y1[2] = 0.0 ;

    int n = 100;
    double[] y2 = new double[n];
    for (int i = 0; i < n; i++)
      y2[i] = Math.sin((double)i/(double)n * Math.PI * 2);

    double[] y3 = new double[15];
    for (int i = 0; i < 15; i++)
      y3[i] = ((double)i / 15.0)*((double)i / 15.0) - 1;

    graphPanel.setAxes(0,1,-1,1);
    graphPanel.plot(y3,Color.red,Graph.BAR);
    graphPanel.plot(y2,Color.magenta,Graph.LINE);
    graphPanel.plot(x,y,Color.blue,Graph.LINE);
    graphPanel.plot(y1,Color.red,Graph.LINE);
    graphPanel.plot(y2,Color.black,Graph.DIAMOND);
    graphPanel.plot("text",0.5,0.2,Color.blue);
  }
  */

}
