package drew.figure;

import java.awt.*;
import math.*;


public class Ellipse extends Figure implements java.io.Serializable
{
  static final long serialVersionUID = 3744437198487682996L;

  int[] px;
  int[] py;
  //Matrix rotation = new Matrix(2,2);
  Matrix rotation;
  Vec scale = new Vec(2);

  boolean filled;
  
  public boolean getFilled() { return filled; }
  
  public void setFilled(boolean b) { filled = b; }


  public void paint(Graphics g)
  {
    if (!live) return;
    if (rotation == null)
      setCovariance(covariance);


    int n = 50;

    Vec x = new Vec(2);

    if (px == null)
      {
	px = new int[n];
	py = new int[n];
      }
    
    Point p = new Point();

    for (int i = 0; i < n; i++)
      {
	double theta = ((double)i)/ ((double)n)*2*Math.PI;
	
	x.set(0, Math.cos(theta));
	x.set(1, Math.sin(theta));
	x = x.scale(scale);
	x = rotation.postMult(x);
	x.scale(stdDevs);
	x = x.add(origin);
	
	transform(x.ref(0), x.ref(1), p);

	px[i] = p.x;
	py[i] = p.y;
      }

    g.setColor(foreground);
    
    if (filled)
      g.fillPolygon(px, py, n);
    else
      g.drawPolygon(px, py, n);

    if (paintCross)
      {

	int sizeCross = 5;
	
	Vec x1 = new Vec(2);
	Vec x2 = new Vec(2);
	
	x1.set(0, -sizeCross);
	x1.set(1, 0);
	
	x2.set(0, sizeCross);
	x2.set(1, 0);
	
	x1 = rotation.postMult(x1);                                    // gross
	x2 = rotation.postMult(x2);
	
	Point po = new Point();
	transform(origin.ref(0), origin.ref(1), po);
	
	Vec to = new Vec(2);
	to.set(0, (double) po.x);
	to.set(1, (double) po.y);
	
	x1 = x1.add(to);
	x2 = x2.add(to);
	
	g.drawLine((int) x1.ref(0), (int) x1.ref(1), (int) x2.ref(0), (int) x2.ref(1));
	
	
	x1.set(0, 0);
	x1.set(1, -sizeCross);
	
	x2.set(0, 0);
	x2.set(1, sizeCross);
	
	x1 = rotation.postMult(x1);
	x2 = rotation.postMult(x2);
	
	x1 = x1.add(to);
	x2 = x2.add(to);
	
	g.drawLine((int) x1.ref(0), (int) x1.ref(1), (int) x2.ref(0), (int) x2.ref(1));
      }

    if (paintLabel)
      {
	g.setFont(font);
	Point po = new Point();
	transform(origin.ref(0), origin.ref(1), po);
	g.drawString(label, po.x, po.y);
      }

  }

  Vec origin = new Vec(2);
  public void setOrigin(Vec origin)
  {
    this.origin = origin;
    //System.out.println("Ellipse.setOrigin() " + origin);
  }
  public Vec getOrigin()
  {
    return origin;
  }


  Matrix covariance = new Matrix(2,2);
  public void setCovariance(Matrix covariance)
  {
    this.covariance = covariance;
    Vec eigenvalues = null;
    try 
      {
	EigenStructure e = new EigenStructure(covariance);
	rotation = e.getEigenvectors();
	eigenvalues = e.getEigenvalues();
      }
    catch (Exception e)
      {
	e.printStackTrace();
      }
    scale.set(0, Math.sqrt(eigenvalues.ref(0)));
    scale.set(1, Math.sqrt(eigenvalues.ref(1)));
  }

  public Matrix getCovariance()
  {
    return covariance;
  }


  double stdDevs = 1.0;
  public void setStdDevs(double stdDevs)
  {
    this.stdDevs = stdDevs;
  }
  public double getStdDevs()
  {
    return stdDevs;
  }

  String label = new String();
  public String getLabel() { return label; }
  public void setLabel(String s) { label = s; }

  boolean paintLabel;
  public boolean getPaintLabel() { return paintLabel; }
  public void setPaintLabel(boolean b) { paintLabel = b; }

 
  boolean paintCross;
  public boolean getPaintCross() { return paintCross; }
  public void setPaintCross(boolean b) { paintCross = b; }
 

  Font font = new Font(null, Font.PLAIN, 10);
  public Font getFont() { return font; }
  public void setFont(Font f) { font = f; }

}


