package drew.figure;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import video.*;

public class MemoryImageFigure extends Figure implements java.io.Serializable
{
  static final long serialVersionUID = 7094102429373213288L;

  protected transient MemoryImageSource memoryImageSource;
  
  // to call createImage:
  JPanel component = new JPanel();

  public void paint(Graphics g)
  {
    if (image != null)
      g.drawImage(image,0,0,component);
  }

  int width;
  public int getWidth() { return width; }
  public void setWidth(int width) 
  { 
    this.width = width;
  }

  int height;
  public int getHeight() { return height; }
  public void setHeight(int height)
  {
    this.height = height;
  }

  transient Image image;
  
  transient ColorModel colorModel;
  //public ColorModel getColorModel() { return colorModel; }
  public void setColorModel(ColorModel colorModel)
  {
    this.colorModel = colorModel;
  }

  protected int numFramesToSkip = 0;
  public int getNumFramesToSkip() { return numFramesToSkip; }
  public void setNumFramesToSkip(int i) { numFramesToSkip = i; }
  
  int framesSinceLastRender = 0;



  public void setPixels(ImagePixels ip)
  {
    if (!renderThisFrame()) 
      return;
    int[] pixels = ip.getPixels();
    setPixels(pixels, ip.getColorModel());
  }

  public void setPixels(FrameGrabberBuffer buffer)
  {
    if (!renderThisFrame()) 
      return;
    int[] pixels = buffer.getPixels(this.pixels);
    setPixels(pixels, buffer.getColorModel());
  }

  public void setPixels(int[] pixels, ColorModel colorModel)
  {
    setColorModel(colorModel);
    setPixels(pixels);
  }

  transient int[] pixels = null;
  
  public void setAxes(double minX, double maxX, double minY, double maxY, Rectangle graphRect)
  {
    super.setAxes(minX,maxX,minY,maxY,graphRect);
    memoryImageSource = null;
    if (pixels != null) 
      setPixels(pixels); // to redo everything for the repaint that is about to happen
  }

  
  protected boolean renderThisFrame()
  {
    if (!live) return false;
    if (framesSinceLastRender < numFramesToSkip)
      {
	framesSinceLastRender++;
	return false;
      }
    return true;
  }


  public void setPixels(int[] pixels)
  {
    framesSinceLastRender = 0;

    if (memoryImageSource == null)
      {
	System.out.println("MemoryImageFigure: creating MemoryImageSource " + width + " " + height);
	memoryImageSource = new MemoryImageSource(width, height, colorModel,
						  pixels, 0, width);
	memoryImageSource.setAnimated(true);
	memoryImageSource.setFullBufferUpdates(true);

	if ((minX == 0) && (maxX == width) && (minY == height) && (maxY == 0)
	    && (graphRect.width == width) && (graphRect.height == height))
	  image = component.createImage(memoryImageSource);
	else
	  {
	    CropImageFilter filter = new CropImageFilter((int)minX, (int) maxY,
							 (int)(maxX - minX), (int)(minY - maxY));
	    ImageProducer producer = new FilteredImageSource(memoryImageSource, filter);

	    image = component.createImage(producer);
	    image = image.getScaledInstance(graphRect.width,graphRect.height,Image.SCALE_FAST);
	  }
	
	this.pixels = pixels;
	
      }
    else if (pixels != this.pixels)
      {
	memoryImageSource.newPixels(pixels, colorModel, 0, width);
	this.pixels = pixels;
      }
    else 
      {
	memoryImageSource.newPixels();
      }
  }


}
