/*
 * This class is meant to be extended for use. In the interest of 
 * speed we leave thread safety up to the caller.
 *
 * Note that when any of the listeners aren't themselves Serializable,
 * serialization of an object of this class will fail.
 * See PropertyChangeSupport for the one way to handle this.
 */

package drew.util;

import java.util.*;
import java.beans.*;
import java.lang.reflect.*;

public class EventListenerList implements java.io.Serializable
{
  // want to have only default constructor so we can
  // extend this class anonymously
  
  // we avoid the use of Vectors during the actual dispatch for speed.
  protected Object[] listeners;  
  protected Vector listenersVector;


  public synchronized void add(Object listener)
  {
    if (listenersVector == null)
      listenersVector = new Vector();
    listenersVector.addElement(listener);
    listeners = new Object[listenersVector.size()];
    listenersVector.copyInto(listeners);
  }

  public synchronized void remove(Object listener)
  {
    listenersVector.removeElement(listener);
    listeners = new Object[listenersVector.size()];
    listenersVector.copyInto(listeners);
  }


  public int getCount() 
  {
    if (listeners == null)
      return 0;
    else
      return listeners.length;
  }

  public boolean hasListeners()
  {
    return getCount() > 0;
  }


  /**
   * Some handy methods
   */
  public void firePropertyChange(Object source, String propertyName, 
				 Object oldValue, Object newValue)
  {
    // We eschew the check oldValue != newValue because it only
    // really makes sense when the properties are primitive types,
    // and besides, its kind of silly anyway.

    if (!hasListeners()) return;

    PropertyChangeEvent e = new PropertyChangeEvent(source,
						    propertyName,
						    oldValue, newValue);

    for (int i = 0; i < listeners.length; i++)
      ((PropertyChangeListener)listeners[i]).propertyChange(e);

  }



}
