package math.pdf;

import math.*;

public class Gaussian extends Pdf
{
  static final long serialVersionUID = -1881531991592312839L;
  
  protected Vec mean, dif, temp;
  protected Matrix C, Cinv;
  protected int d;

  public Gaussian()
  {
    d = 3;
    init();
  }
  
  protected void init()
  {
    Matrix C = new Matrix(d,d);
    C.identity();
    Cinv = new Matrix(d,d);
    mean = new Vec(d);
    dif = new Vec(d);
    temp = new Vec(d);
    setCovariance(C);
  }

  public double evaluate(Vec input)
  {
    Vec.sub(input, mean, dif);
    Matrix.mult(Cinv, dif, temp);
    return Math.exp( (-0.5) * dif.dot(temp)) / scale;
  }

  public double evaluateLog(Vec input)
  {
    Vec.sub(input, mean, dif);
    Matrix.mult(Cinv, dif, temp);
    return (-0.5) * dif.dot(temp) - scale;
  }
  
  public void train()
  {
    trainMean();
    trainCovariance();
  }



  public void trainMean()
  {
    // calculate new value of parameters
    int N = trainingData.size();
    d = trainingData.ref(0).dimension();
    
    //System.out.println("training with " + N + " samples");

    mean = new Vec(d);
    double sumGamma = 0;
    for (int i = 0; i < N; i++)
      {
	Vec x = trainingData.ref(i).copy();
	double thisw;
	if (w == null)
	  thisw = 1.0;
	else
	  thisw = w.ref(i);
	x.scale(thisw);
	Vec.add(mean, x, mean);
	sumGamma += thisw;
      }
    mean.scale(1.0 / sumGamma);

    setMean(mean);
  }


  public void trainCovariance()
  {
    int N = trainingData.size();
    d = trainingData.ref(0).dimension();

    Matrix outer = new Matrix(d,d);
    C = new Matrix(d,d);
    double sumGamma = 0;
    for (int i = 0; i < N; i++)
      {
	Vec x = trainingData.ref(i).copy();
	Vec.sub(x, mean, x);
	x.outer(x, outer);
	double thisw;
	if (w == null)
	  thisw = 1.0;
	else
	  thisw = w.ref(i);
	outer.scale(thisw);
	Matrix.add(C, outer, C);
	sumGamma += thisw;
      }
    C.scale(1.0 / sumGamma);

    setCovariance(C);
  }

  private double scale;
  public int getDimension() { return d; }
  public void setDimension(int d)
  {
    if (this.d == d) return;
    this.d = d;
    init();
  }

  public Matrix getCovariance() { return C; }
  public void setCovariance(Matrix C)
  {
    this.C = C;
    d = C.numRows();
    try
      {
	Cinv = C.inverse();
	double det = C.determinant();
	scale = Math.sqrt( Math.pow( 2*java.lang.Math.PI, d ) * det);
      }
    catch (SingularMatrixException e)
      {
	e.printStackTrace();
      }
    catch (NonSquareMatrixException e)
      {
	e.printStackTrace();
      }
    if (mean.dimension() != d)
      mean = new Vec(d);
    if (input.dimension() != d)
      input = new Vec(d);

    Vec eigenvalues = null;
    try 
      {
	EigenStructure e = new EigenStructure(C);
	rotation = e.getEigenvectors();
	eigenvalues = e.getEigenvalues();
      }
    catch (Exception e)
      {
	e.printStackTrace();
      }
    sqrtEigenvalues = new Vec(eigenvalues.size());
    for (int i = 0; i < eigenvalues.size(); i++)
      sqrtEigenvalues.set(i, Math.sqrt(eigenvalues.ref(i)));

    update();

    if (changeListeners.hasListeners())
      changeListeners.firePropertyChange(this, "covariance", C, C);
    
  }

  Matrix rotation;
  Vec sqrtEigenvalues;


  public Vec getMean() { return mean; }
  public void setMean(Vec mean)
  {
    this.mean = mean;
    d = mean.dimension();
    dif = new Vec(d);
    temp = new Vec(d);
    if (C.numRows() != d)
      {
	Matrix C = new Matrix(d,d);
	C.identity();
	setCovariance(C);
      }
    if (input.dimension() != d)
      input = new Vec(d);
    update();
    
    //System.out.println("set mean");

    if (changeListeners.hasListeners())
      {
	//System.out.println("fire mean change");

	changeListeners.firePropertyChange(this, "mean", mean, mean);
      }
  }

  public static transient java.util.Random randomGenerator;

  public Vec sample()
  {
    if (randomGenerator == null)
      randomGenerator = new java.util.Random();

    Vec x = new Vec(d);
    for (int i = 0; i < d; i++)
      x.set(i, randomGenerator.nextGaussian());

    x = x.scale(sqrtEigenvalues);
    x = rotation.postMult(x);
    x = x.add(mean);
    return x;
  }



}
